﻿using System;
using System.Collections.Concurrent;
using System.IO;
using System.Text.Json;
using System.Threading.Tasks;
using WingServerCommon.Log;

namespace WingUserModule.Common
{
    internal class UserHeartRateManager
    {
        private ConcurrentDictionary<string, UserHeartRateDTO> _users = new ConcurrentDictionary<string, UserHeartRateDTO>();
        private Func<string, bool, bool> OnSetOnlineState;

        public UserHeartRateManager(Func<string, bool, bool> setOnlineState)
        {
            OnSetOnlineState = setOnlineState;
            StartCheckClients();
        }

        /// <summary>
        /// 加入检测集合
        /// </summary>
        /// <param name="token"></param>
        public void AddOrUpdate(string token)
        {
            _users.AddOrUpdate(token, new UserHeartRateDTO(token), (k, v) =>
            {
                v.Activate();
                return v;
            });
        }

        /// <summary>
        /// 设备心跳有效期验证
        /// </summary>
        private void StartCheckClients()
        {
            Logger.WriteLineInfo($"UserHeartRateManager start check clients");
            Task.Run(async () =>
            {
                while (true)
                {
                    await Task.Delay(1 * 60 * 1000);
                    foreach (var token in _users.Keys)
                    {
                        var heartRate = _users[token];
                        heartRate.DeActivate();
                        try
                        {
                            if (heartRate.LeftTime < 0)
                            {
                                OnSetOnlineState?.Invoke(token, false);
                            }
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteLineWarn($"check user token {token} err, {ex}");
                        }
                        finally
                        {
                            if (heartRate.LeftTime < 0)
                            {
                                _users.TryRemove(token, out _);
                            }
                        }
                    }
                }
            });
            Logger.WriteLineInfo($"UserHeartRateManager finished");
        }
    }

    internal class UserHeartRateDTO
    {
        public UserHeartRateDTO(string token)
        {
            Token = token;
        }

        private volatile int _livingTime = 0;
        private readonly int _clientLifecycle = 10; //10 minutes 

        public string Token { get; set; }

        public int LeftTime => _clientLifecycle - _livingTime;

        /// <summary>
        /// Reset the living time.
        /// </summary>
        public virtual void Activate()
        {
            _livingTime = 0;
        }

        /// <summary>
        /// Deactive the session.
        /// </summary>
        public virtual void DeActivate()
        {
            _livingTime++;
        }
    }
}
