using JsonRpcLite.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Request;
using WingServerCommon.Service;
using WingInterfaceLibrary.DTO.IdentityApply;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DB.Request;
using System.IO;
using WingUserModule.Helper;
using WingServerCommon.Log;
using System.Text.RegularExpressions;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IIdentityApplyService
    {
        /// <summary>
        /// 申请身份
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="info">授权信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> ApplyForAsync(ApplyForRequest request)
        {  
            try
            {
                var userCode = await GetClientIdByTokenAsync(request.Token);
                var userInfo = await GetUserInfoAsync(new GetUserInfoRequest { Token = request.Token });
                if (userInfo.RoleCodes != null && userInfo.RoleCodes.Count > 0)
                {
                    ThrowCustomerException(CustomerRpcCode.DuplicateClaim, "Duplicate claim");
                }
                var applyHistory = await GetLastIdentityApplyAsync(new GetLastIdentityApplyRequest { Token = request.Token });
                if (!(applyHistory.ApplyState == ApplyStateEnum.NotApply || applyHistory.ApplyState == ApplyStateEnum.Refused))
                {
                    ThrowCustomerException(CustomerRpcCode.DuplicateClaim, "Duplicate claim");
                }
                if (string.IsNullOrWhiteSpace(request.ApplyRoleCode))
                {
                    ThrowCustomerException(CustomerRpcCode.RoleNoExist, "Role not exist");
                }
                var role = await _roleDBService.FindRoleByCodeAsync(request.ApplyRoleCode);
                if (role == null || string.IsNullOrWhiteSpace(role.RoleCode))
                {
                    ThrowCustomerException(CustomerRpcCode.RoleNoExist, "Role not exist");
                }
                if (role.RoleShowType != RoleShowTypeEnum.ISShow)
                {
                    ThrowCustomerException(CustomerRpcCode.RoleNoApply, "this role can not Apply");
                }
                if (role.RoleQualification == RoleQualificationEnum.ID || role.RoleQualification == RoleQualificationEnum.Both)
                {
                    if (request.IdentityCard == null || request.IdentityCard.Where(w=>!Regex.IsMatch(w,"[a-zA-z]+://[^\\s]*")).Count() > 0)
                    {
                        ThrowCustomerException(CustomerRpcCode.IdentityCardIsRequired, "this identity card is required");
                    }
                }
                if (role.RoleQualification == RoleQualificationEnum.DocLicense || role.RoleQualification == RoleQualificationEnum.Both)
                {
                    if (request.LicenseCard == null || request.LicenseCard.Where(w=>!Regex.IsMatch(w,"[a-zA-z]+://[^\\s]*")).Count() > 0)
                    {
                        ThrowCustomerException(CustomerRpcCode.LicenseCardIsRequired, "this license card  is required");
                    }
                }

                var identityApplyDTO = new IdentityApplyDTO();
                identityApplyDTO.ApplyState = ApplyStateEnum.Applying;
                identityApplyDTO.UserCode = userCode;
                if (request.IdentityCard?.Count > 0)
                {
                    identityApplyDTO.IdentityCard = request.IdentityCard.Select(c => c.ToUrlToken()).ToList();
                }
                if (request.LicenseCard?.Count > 0)
                {
                    identityApplyDTO.LicenseCard = request.LicenseCard.Select(c => c.ToUrlToken()).ToList();
                }
                identityApplyDTO.ApplyRoleCode = request.ApplyRoleCode;
                identityApplyDTO.FieldList = request.FieldList;
                await _identityApplyDBService.InsertIdentityApplyAsync(new CreateIdentityApplyDBRequest
                {
                    Data = identityApplyDTO,
                    ExtensionData = request.ExtensionData
                });
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"user module identity apply failed, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 获取身份申请信息
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="userCode">需要查询的用户编码</param>
        /// <returns></returns>
        public async Task<List<IdentityApplyDTO>> GetIdentityApplysAsync(TokenRequest request)
        {
            try
            {
                var userCode = await GetClientIdByTokenAsync(request.Token);
                var result = await _identityApplyDBService.GetIdentityApplyByUserCodeAsync(userCode);
                return result;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"user module get identity applys failed, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 获取最后一次申请信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IdentityApplyDTO> GetLastIdentityApplyAsync(GetLastIdentityApplyRequest request)
        {
            try
            {
                var userCode = await GetClientIdByTokenAsync(request.Token);
                var result = await _identityApplyDBService.GetIdentityApplyByUserCodeAsync(userCode);
                var applyInfoList = result.OrderByDescending(f => f.CreateTime);
                IdentityApplyDTO applyInfo = null;
                if (!string.IsNullOrWhiteSpace(request.ApplyRoleCode))
                {
                    applyInfo = applyInfoList.FirstOrDefault(f => f.ApplyRoleCode == request.ApplyRoleCode);
                }
                else
                {
                    applyInfo = applyInfoList.FirstOrDefault();
                }
                if (applyInfo == null)
                {
                    applyInfo = new IdentityApplyDTO
                    {
                        ApplyState = ApplyStateEnum.NotApply
                    };
                }
                return applyInfo;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"user module get last identity apply failed, ex:{ex}");
                throw;
            }
        }
    }
}