﻿using System.Collections.Generic;
using System.Threading.Tasks;
using WingUserModule.Processor;
using System;
using System.Text;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Authentication;
using WingUserModule.Entity;
using System.Text.RegularExpressions;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.User;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request;
using WingUserModule.Helper;
using WingServerCommon.Log;
using WingServerCommon.Utilities;
using WingServerCommon.Interfaces.PasswordVerify;
using WingInterfaceLibrary.DTO.User;
using WingServerCommon.Interfaces.Cache;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, ILoginService
    {

        /// <summary>
        /// 检查短信验证码
        /// </summary>
        /// <param name="userPhone">用户手机号</param>
        /// <param name="verifyCode">验证码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CheckSMSVerificationCodeAsync(CheckSMSVerificationCodeRequest request)
        {
            VerifyPhoneValue(request.UserPhone);
            return await _smsService.CheckVerificationCodeAsync(request.UserPhone, request.VerifyCode);
        }

        /// <summary>
        /// 发送短信验证码
        /// </summary>
        /// <param name="userPhone">用户手机号</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SendSMSVerificationCodeAsync(SendSMSVerificationCodeRequest request)
        {
            VerifyPhoneValue(request.UserPhone);
            var smsTemplateId = "1009241";
            return await _smsService.SendMessageAsync(new List<string> { request.UserPhone }, smsTemplateId);
        }

        /// <summary>
        /// 发送邮箱验证码
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SendEmailVerificationCodeAsync(SendEmailVerificationCodeRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.EmailAddress) || !Regex.IsMatch(request.EmailAddress, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
            }
            var verifyCode = GetVerifyCode(request.EmailAddress, 5);
            return await _emailService.SendEmailAsync(request.LanguageCode, new List<string> { request.EmailAddress }, "Simple", verifyCode);
        }

        /// <summary>
        /// 检查邮箱验证码
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <param name="verifyCode">验证码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CheckEmailVerificationCodeAsync(CheckEmailVerificationCodeRequest request)
        {
            VerifyVerificationCodeValue(request.VerifyCode);
            if (string.IsNullOrWhiteSpace(request.EmailAddress) || !Regex.IsMatch(request.EmailAddress, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
                return false;
            }
            if (_emailVerifyCodes.ContainsKey(request.EmailAddress))
            {
                var verifyInfo = _emailVerifyCodes[request.EmailAddress];
                if (verifyInfo.VerifyCode == request.VerifyCode)
                {
                    var result = !_emailVerifyCodes[request.EmailAddress].IsOverdue;
                    if (result)
                    {
                        return await Task.FromResult(result);
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.SMSVerifyCodeExpired, "Verification code expired");
                        return false;
                    }
                }
            }
            ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            return false;
        }

        /// <summary>
        /// 获取邮箱Base64字符串
        /// </summary>
        /// <param name="content"></param>
        /// <returns></returns>
        private string GetBase64EmailString(string content)
        {
            content = Convert.ToBase64String(Encoding.UTF8.GetBytes(content));
            var str = $"=?UTF8?B?{content}?=";
            return str;
        }

        /// <summary>
        /// 获取邮箱验证码（私有）
        /// </summary>
        /// <param name="emailAddress">邮箱地址</param>
        /// <param name="validMinutes">查找时间范围：如5分钟之前</param>
        /// <returns></returns>
        private string GetVerifyCode(string emailAddress, int validMinutes)
        {
            if (_emailVerifyCodes.ContainsKey(emailAddress))
            {
                _emailVerifyCodes.TryRemove(emailAddress, out _);
            }
            var mobileVerifyCode = _emailVerifyCodes.GetOrAdd(emailAddress, new VerifyCodeInfo
            {
                VerifyCode = new Random().Next(1000, 9999).ToString(),
                ValidMinutes = validMinutes,
                CreateTime = DateTime.UtcNow,
            });
            return mobileVerifyCode.VerifyCode;
        }

        /// <summary>
        /// 根据手机找回密码
        /// </summary>
        /// <param name="phone">手机号</param>
        /// <param name="verifyCode">验证码</param>
        /// <param name="newPassword">新密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RetrievePasswordByPhoneAsync(RetrievePasswordByPhoneRequest request)
        {
            var userInfo = await _userDBService.FindUserByPhoneAsync(request.Phone);
            if (userInfo != null)
            {
                var isVerify = await _smsService.CheckVerificationCodeAsync(request.Phone, request.VerifyCode);
                if (isVerify)
                {
                    CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
                    return await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword.ToString());
                }
                return false;
            }
            else
            {
                ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                return false;
            }
        }

        /// <summary>
        /// 根据邮箱找回密码
        /// </summary>
        /// <param name="mail">邮箱地址</param>
        /// <param name="verifyCode">验证码</param>
        /// <param name="newPassword">新密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RetrievePasswordByEmailAsync(RetrievePasswordByEmailRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.Mail) || !Regex.IsMatch(request.Mail, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email formal error!");
            }
            var userInfo = await _userDBService.FindUserByEmailAsync(request.Mail);
            if (userInfo != null)
            {
                if (_emailVerifyCodes.ContainsKey(request.Mail))
                {
                    var isVerify = !_emailVerifyCodes[request.Mail].IsOverdue && _emailVerifyCodes[request.Mail].VerifyCode == request.VerifyCode;
                    if (isVerify)
                    {
                        //密码后期需要加密
                        CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
                        return await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword.ToString());
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                    return false;
                }
            }
            else
            {
                ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                return false;
            }
        }

        /// <summary>
        /// 检查登录类型
        /// </summary>
        /// <param name="anyAccount">登录账号</param>
        /// <returns>登录类型枚举值</returns>
        public async Task<int> CheckLoginTypeAsync(CheckLoginTypeRequest request)
        {
            return await CheckLoginTypeWithExcept(request);
        }

        /// <summary>
        /// 通用登录
        /// </summary>
        /// <param name="anyAccount">用户名/手机/邮箱</param>
        /// <param name="anyCode">密码/验证码</param>
        /// <param name="password">密码</param>
        /// <returns>Token信息</returns>
        public async Task<LoginResult> CommonLoginAsync(CommonLoginRequest request)
        {
            try
            {
                VerifyAccountValue(request.AnyAccount);
                var userLoginInfo = new UserLoginInfo();
                ILoginProcessor processor = null;
                var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
                var enumLoginType = (LoginProcessorTypeEnum)loginType;
                if (string.IsNullOrWhiteSpace(request.Password) && !string.IsNullOrWhiteSpace(request.AnyCode))
                {
                    //如未注册就先注册
                    if (enumLoginType == LoginProcessorTypeEnum.Unregistered)
                    {
                        await CommonSignUpAsync(new CommonSignUpRequest { AnyAccount = request.AnyAccount, AnyCode = request.AnyCode, Password = "121f69e11038b67debf48095df000d14" });//init123456
                        loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
                        enumLoginType = (LoginProcessorTypeEnum)loginType;
                    }
                    processor = LoginProcessorCreator.GetProcessor(enumLoginType);
                    userLoginInfo.LoginType = enumLoginType;
                    switch (userLoginInfo.LoginType)
                    {
                        case LoginProcessorTypeEnum.Phone:
                            var checkPhoneRight = await _smsService.CheckVerificationCodeAsync(request.AnyAccount, request.AnyCode);
                            if (checkPhoneRight)
                            {
                                userLoginInfo.Phone = request.AnyAccount.ToString();
                            }
                            else
                            {
                                ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                            }
                            break;
                        case LoginProcessorTypeEnum.Email:
                            var checkEmailRight = await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode });
                            if (checkEmailRight)
                            {
                                userLoginInfo.Email = request.AnyAccount.ToString();
                            }
                            break;
                    }
                }
                else if (!string.IsNullOrWhiteSpace(request.Password) && string.IsNullOrWhiteSpace(request.AnyCode))
                {
                    if (enumLoginType == LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                    }
                    processor = LoginProcessorCreator.GetProcessor(LoginProcessorTypeEnum.Official);
                    switch (enumLoginType)
                    {
                        case LoginProcessorTypeEnum.Official:
                            userLoginInfo.UserName = request.AnyAccount;
                            userLoginInfo.Password = request.Password;
                            break;
                        case LoginProcessorTypeEnum.Phone:
                            userLoginInfo.Phone = request.AnyAccount.ToString();
                            VerifyPhoneValue(userLoginInfo.Phone);
                            userLoginInfo.Password = request.Password;
                            break;
                        case LoginProcessorTypeEnum.Email:
                            userLoginInfo.Email = request.AnyAccount.ToString();
                            VerifyEmailValue(userLoginInfo.Email);
                            userLoginInfo.Password = request.Password;
                            break;
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.IncorrectPasswordFormat, "Incorrect password format");
                }
                userLoginInfo.Platform = request.Platform;
                userLoginInfo.LoginSource = request.LoginSource;
                string token = string.Empty;
                try
                {
                    token = await processor.DoLoginAsync(userLoginInfo);
                }
                catch (RpcException ex)
                {
                    if (ex.ErrorCode == (int)CustomerRpcCode.PasswordError)
                    {
                        if (!string.IsNullOrWhiteSpace(userLoginInfo.UserCode))
                        {
                            var passwordError = GetGradingProtectionConfigCache("PasswordErrorLimit01");
                            if(passwordError)
                            {
                                var loginLockInfo = _userLoginLock.CheckUserLoginLocked(userLoginInfo.UserCode);
                                if (!loginLockInfo.IsLocked)
                                {
                                    return new LoginResult()
                                    {
                                        LoginState = LoginStateEnum.PasswordIncorrect,
                                        LockRemainingTimes = loginLockInfo?.RemainingTimes
                                    };
                                }
                                ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
                            }
                        }
                    }
                    else if (ex.ErrorCode == (int)CustomerRpcCode.AccountIsLocked)
                    {
                        ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
                    }
                    throw;
                }
                if (!string.IsNullOrWhiteSpace(token))
                {
                    _userHeartRateManager.AddOrUpdate(token);
                    var refreshRequest = new RefreshStaticticRecordsRequest { UserCode = userLoginInfo.UserCode, InExecutor = false };
                    await RefreshStaticticRecordsAsync(refreshRequest);
                }
                var passwordExpired = await VerifyUserPasswordExpire(token);
                return new LoginResult()
                {
                    Token = token,
                    LoginState = LoginStateEnum.Succeed,
                    PasswordExpired = passwordExpired,
                    AccountName = request.AnyAccount,
                };
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"common login failed, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 通用注册
        /// </summary>
        /// <param name="anyAccount">用户名/手机/邮箱</param>
        /// <param name="anyCode">验证码</param>
        /// <param name="password">密码</param>
        /// <returns>是否成功</returns>
        public async Task<bool> CommonSignUpAsync(CommonSignUpRequest request)
        {
            VerifyAccountValue(request.AnyAccount);
            var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
            try
            {
                var loginTypeEnum = (LoginProcessorTypeEnum)loginType;
                switch (loginTypeEnum)
                {
                    case LoginProcessorTypeEnum.Official:
                        ThrowCustomerException(CustomerRpcCode.UserNameRegistered, "UserName is registered");
                        return false;
                    case LoginProcessorTypeEnum.Phone:
                        ThrowCustomerException(CustomerRpcCode.PhoneRegistered, "Phone is registered");
                        return false;
                    case LoginProcessorTypeEnum.Email:
                        ThrowCustomerException(CustomerRpcCode.EmailRegistered, "Email is registered");
                        return false;
                        // case LoginProcessorTypeEnum.Wechat:
                        //     userLoginInfo.ProcessorInfo = new LoginProcessorInfo { UserName = anyAccount, Password = anyCode }
                        //     break;
                }
                string phone = string.Empty, userName = request.AnyAccount, emailAddress = string.Empty;
                if (Regex.IsMatch(request.AnyAccount, "^1[0-9]{10}$"))
                {
                    if (!await CheckSMSVerificationCodeAsync(new CheckSMSVerificationCodeRequest { UserPhone = request.AnyAccount, VerifyCode = request.AnyCode }))
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    phone = request.AnyAccount;
                    userName = CodeCreator.CreateCode("User");
                }
                else if (Regex.IsMatch(request.AnyAccount, "^.+@[a-zA-Z0-9.]+$"))
                {
                    if (!await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode }))
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    emailAddress = request.AnyAccount;
                    userName = CodeCreator.CreateCode("User");
                }
                //if (string.IsNullOrWhiteSpace(request.Password) || !Regex.IsMatch(request.Password, @"^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{6,12}$"))
                if (string.IsNullOrWhiteSpace(request.Password))
                {
                    ThrowCustomerException(CustomerRpcCode.NewPasswordEmpty, "Incorrect password format");
                    return false;
                }
                var userPasswordDTO = new UserPasswordDTO
                {
                    Phone = phone,
                    Email = emailAddress,
                    UserName = userName,
                    SecretPassword = request.Password,
                    PasswordUpdateTime = DateTime.UtcNow,
                    PasswordRecords = new List<string>() { request.Password },
                };
                await _userDBService.InsertUserAsync(new CreateUserInfoDBRequest
                {
                    Data = userPasswordDTO,
                    PlatformDatas = request.HeaderMap,
                    ExtensionData = string.Empty
                });
                return await Task.FromResult(true);
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"common sign up failed, ex:{ex}");
                throw;
            }
        }

        public async Task<bool> VerifyAccountAsync(VerifyAccountRequest request)
        {
            VerifyAccountValue(request.UserName);
            int type = 0;
            if (!string.IsNullOrWhiteSpace(request.UserName))
            {
                if (Regex.IsMatch(request.UserName, _RegexEmail))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                else if (Regex.IsMatch(request.UserName, _RegexPhone))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                else if (Regex.IsMatch(request.UserName, _RegexUserName))
                {
                    type = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.UserName });
                }
                if (type == (int)LoginProcessorTypeEnum.Unregistered)
                {
                    return true;
                }
            }
            return false;
        }

        private async Task<int> CheckLoginTypeWithExcept(CheckLoginTypeRequest request, List<string> exceptUserCodes = null)
        {
            var checkUser = (UserDTO u, List<string> e) => u != null && (e == null || (!e.Contains(u.UserCode)));
            var userInfo = await _userDBService.FindUserByNameAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Official;
            }
            userInfo = await _userDBService.FindUserByPhoneAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Phone;
            }
            userInfo = await _userDBService.FindUserByEmailAsync(request.AnyAccount);
            if (checkUser(userInfo, exceptUserCodes))
            {
                return (int)LoginProcessorTypeEnum.Email;
            }
            return (int)LoginProcessorTypeEnum.Unregistered;
        }

        private async Task<string> GetClientIdByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }


        public async Task<bool> ModifyPasswordAsync(ModifyPasswordRequest request)
        {
            VerifyAccountValue(request.AnyAccount);
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
                return false;
            }
            var loginType = await CheckLoginTypeAsync(new CheckLoginTypeRequest { AnyAccount = request.AnyAccount });
            var enumLoginType = (LoginProcessorTypeEnum)loginType;
            switch (enumLoginType)
            {
                case LoginProcessorTypeEnum.Phone:
                    if (request.AnyAccount != userInfo.Phone)
                    {
                        ThrowCustomerException(CustomerRpcCode.PhoneIllegality, "Phone is illegality");
                        return false;
                    }
                    var checkPhoneRight = await _smsService.CheckVerificationCodeAsync(request.AnyAccount, request.AnyCode);
                    if (!checkPhoneRight)
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    break;
                case LoginProcessorTypeEnum.Email:
                    if (request.AnyAccount != userInfo.Email)
                    {
                        ThrowCustomerException(CustomerRpcCode.EmailIllegality, "Email is illegality");
                        return false;
                    }
                    var checkEmailRight = await CheckEmailVerificationCodeAsync(new CheckEmailVerificationCodeRequest { EmailAddress = request.AnyAccount.ToString(), VerifyCode = request.AnyCode });
                    if (!checkEmailRight)
                    {
                        ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
                        return false;
                    }
                    break;
                case LoginProcessorTypeEnum.Official:
                    if (request.AnyAccount != userInfo.UserName)
                    {
                        ThrowCustomerException(CustomerRpcCode.UserNameIllegality, "UserName is illegality");
                        return false;
                    }
                    if (!await _userDBService.VerificationPasswordAsync(userInfo.UserCode, request.Password))
                    {
                        ThrowCustomerException(CustomerRpcCode.OldPasswordError, "Old password is wrong!");
                        return false;
                    }
                    break;
                default:
                    ThrowCustomerException(CustomerRpcCode.UserNotRegistered, "User not registered");
                    return false;
            }
            //修改密码
            //if (string.IsNullOrWhiteSpace(request.NewPassword) || !Regex.IsMatch(request.NewPassword, @"^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{6,12}$"))
            if (string.IsNullOrWhiteSpace(request.NewPassword))
            {
                ThrowCustomerException(CustomerRpcCode.NewPasswordEmpty, "Incorrect NewPasswrod format");
                return false;
            }
            //验证密码是否重复
            var pwdVerifyReq = new PasswordVerifyModel()
            {
                PasswordRecords = userInfo.PasswordRecords,
                NewPassword = request.NewPassword
            };
            var boolean = _passwordExpireVerify.VerifySamePassword(pwdVerifyReq);
            if (boolean)
            {
                if (_keepPwdTimes <= 0)
                {
                    _keepPwdTimes = 5;
                }
                //没有包含，则添加
                if (userInfo.PasswordRecords.Count >= _keepPwdTimes)
                {
                    userInfo.PasswordRecords.RemoveAt(0);
                }
                if (userInfo.PasswordRecords.Count == 0)
                {
                    userInfo.PasswordRecords.Add(userInfo.SecretPassword);
                }
                userInfo.PasswordRecords.Add(request.NewPassword);
                userInfo.PasswordUpdateTime = DateTime.UtcNow;
            }
            else
            {
                //验证不通过，
                ThrowCustomerException(CustomerRpcCode.UserPasswordRepeatError, "User password repeat error");
            }
            CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(userInfo.UserCode);
            return await _userDBService.UpdateUserPasswordAsync(userInfo.UserCode, request.NewPassword, userInfo.PasswordRecords, userInfo.PasswordUpdateTime);
        }

        public async Task<string> GenerateNewPasswordAsync(GenerateNewPasswordRequest request)
        {
            var randomStr = GenerateRandomStr();
            return await Task.FromResult(randomStr);
        }

        private string GenerateRandomStr()
        {
            char[] constant1 =   
            { 
                'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',   
                'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'   
            };
            var newRandom1 = new System.Text.StringBuilder(52);
            Random rd = new Random();
            for (int i = 0; i < 6; i++)
            {
                newRandom1.Append(constant1[rd.Next(52)]);
            }
            char[] constant2 =   
            {
                '0','1','2','3','4','5','6','7','8','9'
            };
            var newRandom2 = new System.Text.StringBuilder(10);
            for (int i = 0; i < 3; i++)
            {
                newRandom2.Append(constant2[rd.Next(10)]);
            }
            char[] constant3 =   
            {
                '!','@','#','$','%','^','&','*','(',')','_','+'
            };
            var newRandom3 = new System.Text.StringBuilder(12);
            for (int i = 0; i < 3; i++)
            {
                newRandom3.Append(constant3[rd.Next(12)]);
            }
            return $"{newRandom1.ToString()}{newRandom2.ToString()}{newRandom3.ToString()}";
        }
    }
}
