using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Collections.Generic;
using System.Linq;
using WingInterfaceLibrary.DTO.Statistic;
using System;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Utilities.Executors;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.CMSStatistic;
using WingUserModule.Serialization;
using WingUserModule.Helper;
using WingUserModule.Common;
using WingServerCommon.Config;
using WingInterfaceLibrary.Request.User;
using Newtonsoft.Json.Linq;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IUserService
    {
        private DateTime _nextRefreshTime = DateTime.UtcNow;
        private List<StatisticDTO> _statisticConfigs = new List<StatisticDTO>();
        private readonly SequenceExecutor<SaveStatisticInfoDTO> _recordCMSExecutor = new SequenceExecutor<SaveStatisticInfoDTO>("CMSStatisticRecord");

        private void RegisterStatistic()
        {
            if (ConfigurationManager.IsMaster || !ConfigurationManager.IsDistributed)
            {
                Task.Run(async () =>
                {
                    while (true)
                    {
                        await HandleCMSExpiredRecordsAsync();
                        await Task.Delay(1 * 60 * 1000);
                    }
                });
            }
        }

        private async Task HandleCMSExpiredRecordsAsync()
        {
            var expiredRecords = await _statisticDBService.GetExpiredStatisticRecordsAsync(new GetStatisticDBRequest { });
            var userCodes = expiredRecords.Select(x => x.UserCode).Distinct().ToList();
            foreach (var userCode in userCodes)
            {
                var tokenInfo = CacheMaintenance.Instance.Get<ITokensManager>().Where(x => x.ClientId == userCode)?.FirstOrDefault();
                if (tokenInfo == null || tokenInfo.Expiration < DateTime.UtcNow)
                {
                    await _statisticDBService.DeleteStatisticRecordsByUserCodeAsync(new DeleteStatisticRecordsByUserCodeDBRequest { UserCode = userCode });
                }
                else
                {
                    var userRecords = expiredRecords.Where(x => x.UserCode == userCode);
                    foreach (var record in userRecords)
                    {
                        _recordCMSExecutor.Add(OnSaveCMSStaticticInfo, new SaveStatisticInfoDTO { UserCode = record.UserCode, StatisticCode = record.StatisticCode });
                    }
                }
            }
        }

        private async Task<List<StatisticDTO>> GetStatisticInfosAsync()
        {
            if (_nextRefreshTime < DateTime.UtcNow)
            {
                var configs = await _statisticDBService.GetStatisticAsync(new GetStatisticDBRequest());
                if (configs.Count != _statisticConfigs.Count && _statisticConfigs.Count != 0)
                {
                    GetOnlineTokensAsync();//如果增减了配置  自动刷新统计数据
                }
                _statisticConfigs = configs?.ToList();
                _nextRefreshTime = DateTime.UtcNow.AddMinutes(5);
            }
            return _statisticConfigs;
        }

        private async Task GetOnlineTokensAsync()
        {
            var result = await _authenticationService.GetPagedTokensAsync(new GetPagedTokensOnlineRequest()
            {
                PageIndex = 1,
                PageSize = 999999,
                LoginSource = -1,
                LoginFrom = -1,
                IsOnline = "true",
                AccountType = (int)AccountType.User,
            });
            if (result?.PageData?.Count > 0)
            {
                foreach (var item in result.PageData)
                {
                    _recordCMSExecutor.Add(OnSaveCMSStaticticInfo, new SaveStatisticInfoDTO { UserCode = item.ClientId });
                }
            }
        }


        private async Task<bool> FreshStatisticRecordsAsync(IList<StatisticRecordDTO> records, string userCode)
        {
            if (records.Any())
            {
                var configs = GetStatisticInfosAsync().Result;
                foreach (var config in configs)
                {
                    if (!records.Any(x => x.StatisticCode == config.StatisticCode))
                    {
                        await _statisticDBService.DeleteStatisticRecordsByUserCodeAsync(new DeleteStatisticRecordsByUserCodeDBRequest { UserCode = userCode });
                        return true;
                    }
                }
            }
            else
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 更新用户统计数据
        /// </summary>
        /// <param name="userCode"></param>
        /// <param name="inExecutor"></param>
        /// <returns></returns>
        /// 
        public async Task<bool> RefreshStaticticRecordsAsync(RefreshStaticticRecordsRequest request)
        {
            try
            {
                var inExecutor = request.InExecutor;
                var userCode = request.UserCode;
                var configs = GetStatisticInfosAsync().Result;
                foreach (var config in configs)
                {
                    if (inExecutor)
                    {
                        _recordCMSExecutor.Add(OnSaveCMSStaticticInfo, new SaveStatisticInfoDTO { UserCode = userCode, StatisticCode = config.StatisticCode });
                    }
                    else
                    {
                        OnSaveCMSStaticticInfo(new SaveStatisticInfoDTO
                        {
                            UserCode = userCode,
                            StatisticCode = config.StatisticCode,
                        });
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"UserService RefreshStaticticRecordsAsync, refresh user statistic infos failed, ex:{ex}");
            }
            return true;
        }

        private bool OnSaveCMSStaticticInfo(SaveStatisticInfoDTO request)
        {
            try
            {
                var configs = GetStatisticInfosAsync().Result;
                if (!string.IsNullOrWhiteSpace(request.StatisticCode))
                {
                    configs = configs.Where(x => x.StatisticCode == request.StatisticCode).ToList();
                }
                foreach (var config in configs)
                {
                    if (config != null && config.Enabled)
                    {
                        var userCode = request.UserCode;
                        var statisticJsonData = string.Empty;//统计数据json格式字符串
                        switch (config.HandleClass)
                        {
                            case "GetMyPendingStatisticAsync":
                                //我的待办
                                var myPending = _statisticDBService.GetMyPendingStatisticAsync(new GetMyPendingStatisticDBRequest { UserCode = userCode }).Result;
                                statisticJsonData = JsonHelper.SerializerToJson(myPending);

                                break;
                            case "GetMyStandingStatisticAsync":
                                //我的战绩
                                var myStanding = _statisticDBService.GetMyStandingStatisticAsync(new GetMyStandingStatisticDBRequest { UserCode = userCode }).Result;
                                statisticJsonData = JsonHelper.SerializerToJson(myStanding);
                                break;
                            case "GetStatisticMessagesAsync"://消息盒子
                                //查询自己消息
                                var reqMessage = new GetMessagesDBPageRequest()
                                {
                                    ClientId = userCode,
                                    Status = QueryCMSMessageStatusEnum.All,
                                    CurrentPage = 1,
                                    PageSize = 5,
                                    IsFuzzy = true
                                };
                                var messageCollection = _messageDBService.GetMessagesPageDBAsync(reqMessage).Result;
                                var statisticMessages = new List<dynamic>();
                                if (messageCollection != null && messageCollection.PageData != null &&  messageCollection.PageData.Count > 0)
                                {
                                    foreach (var item in messageCollection.PageData)
                                    {
                                        if(item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.RejectApplyConsultationNotification 
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.ConsultationRemindNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.PasswordExpiredWarningNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.AcceptLiveConsultationNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.RejectLiveConsultationNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.ApprovalApplyConsultationNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.InviteeApproveApplyConsultationNotification
                                        || item.NotificationType == WingInterfaceLibrary.Enum.NotificationEnum.NotificationTypeEnum.InviteeRejectApplyConsultationNotification
                                        )
                                        {
                                            var operatorName = string.Empty;
                                            var notifyName = item.NotificationType.ToString();
                                            if(!string.IsNullOrWhiteSpace(item.Content))
                                            {
                                                dynamic jsonData = JObject.Parse(item.Content);
                                                if(jsonData!=null)
                                                {
                                                    operatorName = jsonData.OperatorName+" ";
                                                }
                                            }
                                            statisticMessages.Add(new
                                            {
                                                content = $"{operatorName}{notifyName.Substring(0,1).ToLower()}{notifyName.Substring(1)}",
                                                time = item.NotifyTime,
                                                status = item.ClientInfos?.FirstOrDefault(w=>w.ClientId == userCode)?.IsReaded == true ? "Read" : "Unread"
                                            });
                                        }
                                    }
                                }
                                statisticJsonData = ConvertHelper.ToJson(statisticMessages);//生成消息 Json
                                break;
                            case "GetSystemAnnounceAsync": //系统公告   //查询系统公告
                                var announcements = _announcementDBService.GetAnnouncementsAsync(
                                    new GetAnnouncementsRequest { Top = 3 }
                                ).Result;
                                var statisticAnnounces = new List<dynamic>();
                                foreach (var item in announcements)
                                {
                                    statisticAnnounces.Add(new
                                    {
                                        code = item.Code,
                                        announcementType = item.AnnouncementType,
                                        languageContents = item.LanguageConfigs.Select(s=> new{
                                            content = s.Content,
                                            languageCode = s.Language,
                                            title = s.Title
                                        })
                                    });
                                }
                                statisticJsonData = ConvertHelper.ToJson(statisticAnnounces);//生成公告 Json
                                break;
                        }
                        if (!string.IsNullOrEmpty(statisticJsonData))
                        {
                            var nextHandleTime = CronHelper.CronToNextDateTime(config.IntervalSetting, DateTime.UtcNow);//解析到的时间都是固定的时间点，而不是直接加间隔时间
                            if (nextHandleTime == default)
                            {
                                nextHandleTime = DateTime.UtcNow.AddMinutes(5);
                            }
                            var result = _statisticDBService.SaveStatisticRecordAsync(new SaveStatisticRecordDBRequest
                            {
                                StatisticCode = config.StatisticCode,
                                StatisticName = config.Name,
                                UserCode = userCode,
                                NextHandleTime = nextHandleTime,
                                StatisticJsonData = statisticJsonData,
                                StatisticsPlatform = StatisticsPlatformEnum.CMS,
                                ServerHost = ConfigurationManager.Host
                            }).Result;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"UserService OnSaveCMSStaticticInfo, save user statistic info failed, ex:{ex}");
            }
            return true;
        }
    }

    public class SaveStatisticInfoDTO
    {
        //
        // 摘要:
        //     图表配置编码
        public string StatisticCode { get; set; }
        //
        // 摘要:
        //    用户编码
        public string UserCode { get; set; }
    }
}