﻿using System.Threading.Tasks;
using System;
using WingInterfaceLibrary.Interface;
using WingUserModule.Entity;
using System.Collections.Generic;
using System.Linq;
using WingInterfaceLibrary.Request.User;
using RemoveUsersFromOrganizationRequest = WingInterfaceLibrary.Request.User.RemoveUsersFromOrganizationRequest;
using SetUserOrganizationInfoRequest = WingInterfaceLibrary.Request.User.SetUserOrganizationInfoRequest;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Request;
using System.Collections.Concurrent;
using WingServerCommon.Service;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingUserModule.Processor;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Result.UserFeature;
using WingInterfaceLibrary.Request.DBRequest;
using WingUserModule.Helper;
using WingInterfaceLibrary.DTO.Rank;
using WingInterfaceLibrary.Request.Authentication;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingUserModule.Common;
using System.Text.RegularExpressions;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Interfaces.PasswordVerify;
using WingServerCommon.Interfaces.Login;
using WingInterfaceLibrary.Notifications;
using WingInterfaceLibrary.DTO.Schedule;
using WingInterfaceLibrary.DTO.GradingProtection;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Internal.Interface;
using WingInterfaceLibrary.Internal.Request;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IUserService
    {
        private IAdminDBService _adminDBService;
        private IUserDBService _userDBService;
        private IAuthenticationService _authenticationService;
        private IIdentityApplyDBService _identityApplyDBService;
        private IStorageDBService _storageDBService;
        private IOrganizationDBService _organizationDBService;
        private IDeviceInfoDBService _deviceInfoDBService;
        private IRoleDBService _roleDBService;
        private IRankDBService _rankDBService;
        private ISMSService _smsService;
        private IEmailService _emailService;
        private IPatientInfoDBService _patientInfoDBService;
        private IUserFeaturesDBService _userFeaturesDBService;
        private IUserAuthorityGroupsDBService _userAuthorityGroupsDBService;
        private ConcurrentDictionary<string, VerifyCodeInfo> _emailVerifyCodes = new ConcurrentDictionary<string, VerifyCodeInfo>();

        private IRecordsInfoDBService _recordsInfoDBService;

        private IStatisticDBService _statisticDBService;
        private ICMSTemplateDBService _cmsTemplateDBService;
        private IMessageDBService _messageDBService;
        private IAnnouncementDBService _announcementDBService;
        private int _heartRateSeconds;
        private int _defaultQueryDays;
        private UserHeartRateManager _userHeartRateManager;

        private UserLoginLock _userLoginLock;


        private string _RegexPhone = "^1(3[0-9]|4[01456879]|5[0-35-9]|6[2567]|7[0-8]|8[0-9]|9[0-35-9])\\d{8}$";

        private string _RegexEmail = "^[A-Za-z0-9-._]+@[A-Za-z0-9-]+(.[A-Za-z0-9]+)?(.[A-Za-z]{2,6})$";

        private string _RegexUserName = @"^[A-Za-z0-9]{2,20}$";
        private INotificationService _notificationService;
        private UserPasswordExpireVerify _passwordExpireVerify;
        private int _keepPwdTimes;
        private string _webSocketUrl = string.Empty;
        private int _liveConsultationRateSeconds = 0;
        private string _cmsUrl = string.Empty;

        /// <summary>
        /// Init service
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _adminDBService = GetProxy<IAdminDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _storageDBService = GetProxy<IStorageDBService>();
            _organizationDBService = GetProxy<IOrganizationDBService>();
            _deviceInfoDBService = GetProxy<IDeviceInfoDBService>();
            _roleDBService = GetProxy<IRoleDBService>();
            _rankDBService = GetProxy<IRankDBService>();
            _userDBService = GetProxy<IUserDBService>();
            _positionDBService = GetProxy<IPositionDBService>();
            _identityApplyDBService = GetProxy<IIdentityApplyDBService>();
            _smsService = GetProxy<ISMSService>();
            _emailService = GetProxy<IEmailService>();
            _patientInfoDBService = GetProxy<IPatientInfoDBService>();
            _userFeaturesDBService = GetProxy<IUserFeaturesDBService>();
            _userAuthorityGroupsDBService = GetProxy<IUserAuthorityGroupsDBService>();
            _recordsInfoDBService = GetProxy<IRecordsInfoDBService>();
            _statisticDBService = GetProxy<IStatisticDBService>();
            _cmsTemplateDBService = GetProxy<ICMSTemplateDBService>();
            _messageDBService = GetProxy<IMessageDBService>();
            _announcementDBService = GetProxy<IAnnouncementDBService>();
            LoginProcessorCreator.InitProcessorCreator(_userDBService, _authenticationService, checkedLoginIsLocked);
            _heartRateSeconds = ConfigurationManager.GetParammeter<IntParameter>("User", "HeartRateSeconds").Value;
            CacheMaintenance.Instance.Get<IUserInfoManager>().RegisterDBFunction(LoadUserModule, LoaUserModules);
            CacheMaintenance.Instance.Get<IGradingProtectionConfigManager>().RegisterDBFunction(LoadGradingProtectionConfigModule, LoadGradingProtectionConfigModules);
            _defaultQueryDays = ConfigurationManager.GetParammeter<IntParameter>("Live", "DefaultQueryDays").Value;
            var loginLockTimes = ConfigurationManager.GetParammeter<IntParameter>("Authentication", "LoginLockTimes").Value;
            _userLoginLock = new UserLoginLock(loginLockTimes);
            _userLoginLock.LoginLockInfoChanged += LoginLockInfoChanged;
            _userHeartRateManager = new UserHeartRateManager(SetOnlineState);
            _notificationService = GetProxy<INotificationService>();
            _keepPwdTimes = ConfigurationManager.GetParammeter<IntParameter>("Authentication", "KeepPwdTimes").Value;
            _passwordExpireVerify = new UserPasswordExpireVerify(new Func<IList<string>, IList<PasswordVerifyModel>>(LoadUserDataByCodes), true);
            _passwordExpireVerify.SendNoticeEvent += OnSendNoticeEvent;
            _webSocketUrl = ConfigurationManager.GetParammeter<StringParameter>("Notification", "WebSocketUrl").Value;
            _liveConsultationRateSeconds = ConfigurationManager.GetParammeter<IntParameter>("Live", "HeartRateSeconds").Value;
            _cmsUrl = ConfigurationManager.GetParammeter<StringParameter>("Management", "CMSUrl").Value;
        }

        public override void LoadAfterRegister()
        {
            try
            {
                RegisterStatistic();
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserService LoadAfterRegister err, {ex}");
            }
        }

        /// <summary>
        /// 发送通知委托         
        /// </summary>
        /// <param name="request">用户登录请求实体</param>
        /// <returns></returns>
        private void OnSendNoticeEvent(Object sender, SendNoticeArgs args)
        {
            if (args != null && args.VerifyList != null && args.VerifyList.Count > 0)
            {
                foreach (var item in args.VerifyList)
                {
                    if (item.SurplusTime == 0)
                    {
                        //正常，不需要预警
                        continue;
                    }
                    //发送通知
                    var userPwdNotice = new PasswordExpiredWarningNotification()
                    {
                        SurplusTime = item.SurplusTime,
                    };
                    //Logger.WriteLineInfo($"Notify PasswordExpiredWarningNotification user {item.ClientId}");
                    var sendNotificationRequest = new SendNotificationRequest() { 
                        ClientId = item.ClientId, 
                        Message = userPwdNotice,
                        JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(userPwdNotice),
                        NotificationType = userPwdNotice.NotificationType,
                        TransactionType = TransactionTypeEnum.Session,
                        RelevanceCode = item.ClientId,
                        ReceiverType = ApplicantTypeEnum.Client
                    };
                    var result = _notificationService.PostMessageAsync(sendNotificationRequest).GetAwaiter().GetResult();
                    if (result)
                    {
                        Logger.WriteLineInfo($"Send update user password notice success, user {item.ClientId}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"Send update user password notice fail, user {item.ClientId}");
                    }
                }
            }
        }

        /// <summary>
        /// 登录验证密码是否过期
        /// </summary>
        /// <param name="token">token字符串</param>
        /// <returns></returns>
        private async Task<bool> VerifyUserPasswordExpire(string token)
        {
            var passwordOverdue = GetGradingProtectionConfigCache("PasswordOverdueLimit01");
            if(!passwordOverdue)
            {
                return false;
            }
            var userCode = await GetClientIdByTokenAsync(token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
                return false;
            }
            //验证密码是否过期
            bool passwordExpired = false;
            var verifyReq = new PasswordVerifyModel()
            {
                ClientId = userInfo.UserCode,
                PasswordLastUpdateTime = userInfo.PasswordUpdateTime
            };
            var verifyResult = _passwordExpireVerify.VerifyPasswordExpire(verifyReq);
            if (verifyResult.SurplusTime >= 0)
            {
                //表示未过期，不做处理
            }
            else
            {
                //表示已过期
                passwordExpired = true;
            }
            return passwordExpired;
        }

        /// <summary>
        /// 加载User密码数据信息
        /// </summary>
        /// <param name="request">用户codes列表</param>
        /// <returns></returns>
        private IList<PasswordVerifyModel> LoadUserDataByCodes(IList<string> codes)
        {
            IList<PasswordVerifyModel> resultList = new List<PasswordVerifyModel>();
            if (codes?.Count > 0)
            {
                var userPwdList = _userDBService.FindUsersByUserCodesAsync(codes.ToList()).GetAwaiter().GetResult();
                if (userPwdList?.Count > 0)
                {
                    resultList = userPwdList.FindAll(c => !string.IsNullOrEmpty(c.SecretPassword)).Select(d => new PasswordVerifyModel()
                    {
                        ClientId = d.UserCode,
                        PasswordLastUpdateTime = d.PasswordUpdateTime
                    }).ToList();
                }
            }
            return resultList;
        }

        private WingServerCommon.Interfaces.Cache.CacheUserDTO LoadUserModule(string code)
        {
            return LoaUserModules(new List<string> { code }).FirstOrDefault();
        }

        private List<WingServerCommon.Interfaces.Cache.CacheUserDTO> LoaUserModules(IList<string> codes)
        {
            List<UserDTO> dtos = _userDBService.FindUserListByUserCodesAsync(codes.ToList()).Result;
            return dtos.MappingTo<List<WingServerCommon.Interfaces.Cache.CacheUserDTO>>();
        }

        private WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO LoadGradingProtectionConfigModule(string code)
        {
            return LoadGradingProtectionConfigModules(new List<string> { code }).FirstOrDefault();
        }

        private List<WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO> LoadGradingProtectionConfigModules(IList<string> codes)
        {
            List<GradingProtectionConfigDTO> dtos = _adminDBService.FindGradingProtectionConfigListAsync(codes.ToList()).Result;
            return dtos.MappingTo<List<WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO>>();
        }

        private async void LoginLockInfoChanged(object sender, LoginLockEventArgs args)
        {
            try
            {
                if (args != null)
                {
                    var passwordError = GetGradingProtectionConfigCache("PasswordErrorLimit01");
                    if(!passwordError)
                    {
                        return;
                    }
                    var result = await _userDBService.UpdateUserLoginLockInfoAsync(
                      args.Code,
                      new LoginLockInfoDTO()
                      {
                          LoginDate = args.LoginDate,
                          Times = args.Times
                      });
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserService LoginLockInfoChanged err, {ex}");
            }

        }
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg);
        }

        /// <summary>
        /// 手机号验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyPhoneValue(string value)
        {
            if (!string.IsNullOrWhiteSpace(value) && !Regex.IsMatch(value, _RegexPhone))
            {
                ThrowCustomerException(CustomerRpcCode.PhoneIncorrect, "Phone incorrect!");
            }
        }
        /// <summary>
        /// Email验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyEmailValue(string value)
        {
            if (!string.IsNullOrWhiteSpace(value) && !Regex.IsMatch(value, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email incorrect!");
            }
        }

        /// <summary>
        /// 账号名称验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyAccountValue(string value)
        {
            if (string.IsNullOrWhiteSpace(value))
            {
                ThrowCustomerException(CustomerRpcCode.AccountIsEmpty, "Account is empty");
            }
            else if (value.Length < 2 || value.Length > 50)
            {
                ThrowCustomerException(CustomerRpcCode.AccountLengthError, "Account length incorrect");
            }
        }

        /// <summary>
        /// 验证 验证码
        /// </summary>
        /// <param name="value"></param>
        public void VerifyVerificationCodeValue(string value)
        {
            if (!string.IsNullOrWhiteSpace(value) && value.Length < 4 || value.Length > 6)
            {
                ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            }
        }

        /// <summary>
        /// 客户端心跳
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> HeartRateAsync(TokenRequest request)
        {
            _userHeartRateManager.AddOrUpdate(request.Token);
            var result = SetOnlineState(request.Token, true);
            return result;
        }

        private bool SetOnlineState(string token, bool isOnline)
        {
            return _authenticationService.SetOnlineStateAsync(new SetOnlineStateRequest { Token = token, IsOnline = isOnline }).Result;
        }

        /// <summary>
        /// 获取用户信息
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <returns>是否成功</returns>
        public async Task<UserDTO> GetUserInfoAsync(GetUserInfoRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new FindUserRequest();
            dbRequest.Code = userCode;
            var userDTO = await _userDBService.FindUserAsync(dbRequest);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var userInfo = userDTO.MappingTo<UserDTO>();
            if (!string.IsNullOrEmpty(userInfo.OrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.OrganizationCode);
                if (orgInfo != null)
                {
                    userInfo.OrganizationName = orgInfo.OrganizationName;
                }
            }
            if (!string.IsNullOrEmpty(userInfo.RootOrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.RootOrganizationCode);
                if (orgInfo != null)
                {
                    userInfo.RootOrganizationName = orgInfo.OrganizationName;
                }
            }
            return userInfo;
        }

        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="userInfo">用户信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> AlterUserInfoAsync(AlterUserInfoRequest request)
        {
            try
            {
                var userCode = await GetClientIdByTokenAsync(request.Token);
                var findUserDBRequest = new FindUserRequest();
                findUserDBRequest.Code = userCode;
                var srcUserInfoDTO = await _userDBService.FindUserAsync(findUserDBRequest);
                var org = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = request.OrganizationCode });
                var allDevice = await _deviceInfoDBService.FindDeviceInfoListByOrganizationCodeAsync(request.OrganizationCode);
                //如果是部门
                if (org?.RootCode != org?.OrganizationCode)
                {
                    var rootDevices = await _deviceInfoDBService.FindDeviceInfoListByOrganizationCodeAsync(org?.RootCode);
                    allDevice.AddRange(rootDevices);
                }
                var sharedDevices = allDevice.Where(e => e.IsAutoShared == true).Select(f => f.DeviceCode).Distinct().ToList();
                if (srcUserInfoDTO.BindDevices != null && srcUserInfoDTO.BindDevices.Count > 0)
                {
                    sharedDevices = sharedDevices.Except(srcUserInfoDTO.BindDevices).ToList();
                }
                string oldUrl = string.Empty;
                if (!string.IsNullOrWhiteSpace(request.Phone))
                {
                    //解密验证手机号是否合法
                    VerifyPhoneValue(request.Phone);
                    var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Phone.ToString() }, new List<string> { userCode });
                    if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.PhoneBinded, "Phone repeat binding");
                    }
                }
                srcUserInfoDTO.Phone = request.Phone;
                if (!string.IsNullOrWhiteSpace(request.Email))
                {
                    VerifyEmailValue(request.Email.ToString());
                    var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Email.ToString() }, new List<string> { userCode });
                    if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.EmailBinded, "EMail repeat binding");
                    }
                }
                srcUserInfoDTO.Email = request.Email;
                // delegate get organization method
                var getOrganizationDTO = async (string code) => string.IsNullOrWhiteSpace(code)
                ? null
                : await _organizationDBService.GetOrganizationDetailAsync
                (
                    new GetOrganizationDBRequest
                    {
                        OrganizationCode = code
                    }
                );
                OrganizationDTO newOrgModel = await getOrganizationDTO(request.OrganizationCode);
                OrganizationDTO oldOrgModel = await getOrganizationDTO(srcUserInfoDTO.OrganizationCode);
                if (newOrgModel?.RootCode != oldOrgModel?.RootCode)//换机构
                {
                    //清除角色、设备分享关系、机构负责人、病人创建人、角色申请记录等业务字段,绑定新的自动分享设备
                    CleanBindInfo(srcUserInfoDTO, sharedDevices);
                }
                else
                {
                    srcUserInfoDTO.BindDevices.AddRange(sharedDevices);
                    //换部门
                    if (newOrgModel?.OrganizationCode != oldOrgModel?.OrganizationCode)
                    {
                        srcUserInfoDTO.PositionCodes = new List<string>();
                    }
                }
                srcUserInfoDTO.NickName = request.NickName;
                if (!string.IsNullOrEmpty(request.NickName))
                {
                    VerifyAccountValue(request.NickName);
                }
                srcUserInfoDTO.FullName = request.FullName;
                srcUserInfoDTO.OrganizationCode = request.OrganizationCode;
                if (!string.IsNullOrEmpty(request.OrganizationCode))
                {
                    VerifyAccountValue(request.OrganizationCode);
                }
                srcUserInfoDTO.UpdateTime = DateTime.UtcNow;
                if (!string.IsNullOrEmpty(request.HeadImageUrl))
                {
                    if (!Regex.IsMatch(request.HeadImageUrl, @"[a-zA-z]+://[^\s]*"))
                    {
                        ThrowCustomerException(CustomerRpcCode.HeadImageUrlNotValid, "HeadImageUrl is not valid");
                    }
                    srcUserInfoDTO.HeadImageUrl = request.HeadImageUrl.ToUrlToken();
                }
                var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), request.ExtensionData, true);
                return result;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"user module alter user info failed, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 重置用户在机构的数据
        /// </summary>
        /// <param name="srcUserInfoDTO"></param>
        private void CleanBindInfo(UserDTO srcUserInfoDTO, List<string> sharedDevices)
        {
            //清除角色、设备分享关系、岗位身份、机构负责人、病人创建人、病人操作人、角色申请记录等业务字段
            srcUserInfoDTO.RoleCodes = new List<string>();
            srcUserInfoDTO.BindDevices = sharedDevices;
            srcUserInfoDTO.AuthorityGroups = new List<string>();
            srcUserInfoDTO.RankCodes = new List<string>();
            srcUserInfoDTO.PositionCodes = new List<string>();
            //查到该用户作为机构负责人的机构
            var allOrganizationFilter = new Dictionary<string, string>();
            allOrganizationFilter.Add("Director", srcUserInfoDTO.UserCode);
            var allOrganizations = _organizationDBService.GetOrganizationPageAsync((new PageFilterRequest { CurrentPage = 1, PageSize = 999999, Filter = allOrganizationFilter, IsFuzzy = false }));
            var needOrgs = allOrganizations.Result.PageData;
            foreach (var orgInfo in needOrgs)
            {
                orgInfo.Directors.Remove(srcUserInfoDTO.UserCode);
                _organizationDBService.SaveOrganizationAsync(orgInfo.OrganizationCode, orgInfo);
            }
            var patientInfoList = _patientInfoDBService.GetPatientsByOperatorAsync(new GetPatientsByOperatorRequest { UserCode = srcUserInfoDTO.UserCode });
            if (patientInfoList.Result.Count > 0)
            {
                foreach (var item in patientInfoList.Result)
                {
                    item.CreatorCode = "";
                    item.UpdateUsers?.Remove(srcUserInfoDTO.UserCode);
                    _patientInfoDBService.UpdatePatientsAsync(item.PatientCode, item, null, true);
                }
            }
            //角色申请记录
            var applys = _identityApplyDBService.GetIdentityApplyByUserCodeAsync(srcUserInfoDTO.UserCode);
            if (applys != null && applys.Result != null && applys.Result.Count > 0)
            {
                foreach (var item in applys.Result)
                {
                    //以后都要改成软删除  查询处需做调整
                    _identityApplyDBService.DeleteIdentityApplyByCodeAsync(item.IdentityApplyCode);
                }
            }
            //检查记录的创建人
            var recordFilter = new Dictionary<string, string>();
            recordFilter.Add("CreatorCode", srcUserInfoDTO.UserCode);
            var recordList = _recordsInfoDBService.FindRecordInfoPagesAsync(new FindRecordInfoPagesRequest { CurrentPage = 1, PageSize = 999999, Filter = recordFilter, IsFuzzy = true });
            if (recordList.Result.DataCount > 0)
            {
                foreach (var item in recordList.Result.PageData)
                {
                    item.CreatorCode = "";
                    _recordsInfoDBService.UpdateRecordInfoByCodeAsync(item.RecordCode, item, null, true);
                }
            }
        }

        /// <summary>
        /// 个人中心-修改头像昵称
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="userInfo">用户信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> AlterPersonInfoAsync(AlterPersonInfoRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.NickName) && string.IsNullOrEmpty(request.HeadImageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.NickNameAndHeadImageUrlEmpty, "NickName and HeadImageUrl Empty error");
            }
            var srcUserInfoDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (!string.IsNullOrEmpty(request.NickName))
            {
                if (request.NickName.Length > 50)
                {
                    ThrowCustomerException(CustomerRpcCode.NicknameTooLong, "Nickname is too long");
                }
                srcUserInfoDTO.NickName = request.NickName;
            }
            if (!string.IsNullOrEmpty(request.HeadImageUrl))
            {
                if (!Regex.IsMatch(request.HeadImageUrl, @"[a-zA-z]+://[^\s]*"))
                {
                    ThrowCustomerException(CustomerRpcCode.HeadImageUrlNotValid, "HeadImageUrl is not valid");
                }
                srcUserInfoDTO.HeadImageUrl = request.HeadImageUrl.ToUrlToken();
            }
            srcUserInfoDTO.UpdateTime = DateTime.UtcNow;
            var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), string.Empty);
            return result;
        }

        /// <summary>
        /// 获取用户信息列表
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>用户信息列表</returns>
        public async Task<List<UserExtendDTO>> GetUserListAsync(GetUserListRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new GetUserListDBRequest
            {
                Keyword = request.Keyword,
                OrganizationCode = request.OrganizationCode,
                OrganizationQueryType = request.OrganizationQueryType,
                RankCode = request.RankCode,
                PositionCode = request.PositionCode,
                ExceptUsers = new List<string>(),
                RoleCodes = request.RoleCodes
            };
            if (request.ExceptSelf)
            {
                dbRequest.ExceptUsers.Add(userCode);
            }
            var users = await _userDBService.GetUserListAsync(dbRequest);
            var result = users.MappingTo<List<UserExtendDTO>>();
            var findAuthenticationRolesRequest = new FindAuthenticationRolesRequest { Language = request.Language };
            var roles = await FindAuthenticationRolesAsync(findAuthenticationRolesRequest);
            foreach (var item in result)
            {
                if (item.RoleCodes != null && item.RoleCodes.Count > 0)
                {
                    item.RoleName = roles.FirstOrDefault(x => x.RoleCode == item.RoleCodes[0])?.RoleName;
                }
            }
            return result;
        }

        /// <summary>
        /// 从当前用户组织中删除用户
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>删除数量</returns>
        public async Task<bool> RemoveUsersFromOrganizationAsync(RemoveUsersFromOrganizationRequest request)
        {
            var users = await _userDBService.FindUserListByUserCodesAsync(request.UserCodes);
            foreach (var item in users)
            {
                item.OrganizationCode = "";
                await _userDBService.UpdateUserAsync(item.UserCode, item.MappingTo<UserPasswordDTO>(), "", true);
            }
            return true;
        }

        /// <summary>
        /// 设置用户组织信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns></returns>
        public async Task<bool> SetUserOrganizationInfoAsync(SetUserOrganizationInfoRequest request)
        {
            var userInfo = await _userDBService.FindUserByCodeAsync(request.UserCode);
            if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                userInfo.OrganizationCode = request.OrganizationCode;
            }
            if (request.PositionCodes != null && request.PositionCodes.Count > 0)
            {
                userInfo.PositionCodes = request.PositionCodes;
            }
            if (request.RankCodes != null && request.RankCodes.Count > 0)
            {
                userInfo.RankCodes = request.RankCodes;
            }
            if (request.RoleCodes != null && request.RoleCodes.Count > 0)
            {
                userInfo.RoleCodes = request.RoleCodes;
            }
            return await _userDBService.UpdateUserAsync(userInfo.UserCode, userInfo.MappingTo<UserPasswordDTO>(), string.Empty);
        }

        private async Task<string> AddFileTokenInfo(string url)
        {
            var result = await _storageDBService.StorageAsync(url);
            return result?.Token;
        }
        private async Task<string> GetFileUrlByToken(string token)
        {
            var result = await _storageDBService.GetStorageRouteAsync(token);
            return result?.StorageRoute;
        }

        public async Task<PageResult<ShareDeviceUserDTO>> GetShareDeviceUsersAsync(GetShareDeviceUsersPageRequest request)
        {
            var deviceCode = request.DeviceCode;
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //检查设备编码不为空
            if (string.IsNullOrWhiteSpace(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Required parameter:DeviceCode cannot be empty");
            }
            //查询设备对象
            var deviceDb = await _deviceInfoDBService.FindDeviceInfoByCodeAsync(deviceCode);
            if (deviceDb == null)
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var shareDeviceUsers = new List<ShareDeviceUserDTO>();
            var filter = new Dictionary<string, string>();
            filter.Add("DeviceCode", deviceCode);
            var userListResult = await _userDBService.FindUserPagesCustomAsync(new PageFilterRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                IsFuzzy = true
            });
            if (userListResult.PageData.Count > 0)
            {
                var organizations = await _organizationDBService.FindOrganizationListByCodesAsync(userListResult.PageData.Select(u => u.OrganizationCode).ToList());
                var rootOrganizations = await _organizationDBService.FindOrganizationListByCodesAsync(organizations.Select(o => o.RootCode).ToList());
                var ranks = await GetUserRanks(userListResult.PageData);
                foreach (var item in userListResult.PageData)
                {
                    var rankNames = ranks.Where(v => item.RankCodes.Contains(v.RankCode)).Select(c => c.RankName).ToList();
                    var organization = organizations.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                    var rootOrganization = rootOrganizations.FirstOrDefault(r => r.OrganizationCode == organization?.RootCode);
                    var shareDeviceUserDTO = new ShareDeviceUserDTO()
                    {
                        UserCode = item.UserCode,
                        FullName = item.FullName,
                        Phone = item.Phone,
                        HeadImageUrl = item.HeadImageUrl,
                        RankNames = rankNames.Select(f => f.ToString()).ToList(),
                        RootOrganizationCode = rootOrganization?.RootCode,
                        RootOrganizationName = rootOrganization?.OrganizationName
                    };
                    shareDeviceUsers.Add(shareDeviceUserDTO);
                }
            }
            var result = new PageResult<ShareDeviceUserDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = userListResult.DataCount;
            result.PageData = shareDeviceUsers;
            return result;
        }

        /// <summary>
        /// 根据登录用户获取权限列表
        /// </summary>
        /// <param name="request">token</param>
        /// <returns>权限列表</returns>
        public async Task<List<UserFeatureInfoResult>> GetUserFeatureListAsync(TokenRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var result = new List<UserFeatureInfoResult>();
            var allFeaturesResult = await _userFeaturesDBService.FindUserFeaturePagesAsync(new PageFilterRequest { CurrentPage = 1, PageSize = 9999, Filter = new Dictionary<string, string>(), IsFuzzy = true });
            var allFeatures = allFeaturesResult.PageData.MappingTo<List<UserFeatureInfoResult>>();
            if (!string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                //当前用户是所在机构的负责人则返回所有权限
                if (userInfo.IsDirector)
                {
                    return allFeatures;
                }
                else if (userInfo.RoleCodes != null && userInfo.RoleCodes.Count > 0)//当前用户的角色（目前只支持单个角色）
                {
                    var roleDB = await _roleDBService.FindRoleByCodeAsync(userInfo.RoleCodes[0]);
                    var userAuthorityGroupDB = await _userAuthorityGroupsDBService.FindUserAuthorityGroupByCodeAsync(roleDB?.UserGroupCode);
                    result = allFeatures.Where(f => userAuthorityGroupDB.Features.Contains(f.FeatureCode)).ToList();
                }
            }
            //没有权限则给普通用户的权限
            if (result.Count == 0)
            {
                var roleDB = await _roleDBService.FindRoleByCodeAsync("Role_GeneralUser");
                var userAuthorityGroupDB = await _userAuthorityGroupsDBService.FindUserAuthorityGroupByCodeAsync(roleDB?.UserGroupCode);
                result = allFeatures.Where(f => userAuthorityGroupDB.Features.Contains(f.FeatureCode)).ToList();
            }
            var resultList = result.MappingTo<List<UserFeatureInfoResult>>();
            //半选特殊处理
            foreach (var item in result)
            {
                GetUserFatherFeature(item, resultList, allFeatures);
            }
            return resultList;
        }

        private void GetUserFatherFeature(UserFeatureInfoResult item, List<UserFeatureInfoResult> list, List<UserFeatureInfoResult> allFeatures)
        {
            if (string.IsNullOrWhiteSpace(item.FatherCode))
            {
                return;
            }
            var fatherItem = allFeatures.FirstOrDefault(x => x.FeatureCode == item.FatherCode);
            if (!list.Select(x => x.FeatureCode).Contains(fatherItem.FeatureCode))
            {
                list.Add(fatherItem);
                GetUserFatherFeature(fatherItem, list, allFeatures);
            }
        }


        /// <summary>
        /// 根据用户编码获取详情
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>用户信息</returns>
        public async Task<UserInfoByCodeDTO> GetUserByCodeAsync(GetUserByCodeRequest request)
        {
            var dbRequest = new FindUserRequest();
            dbRequest.Code = request.UserCode;
            var userDTO = await _userDBService.FindUserAsync(dbRequest);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var userInfo = userDTO.MappingTo<UserInfoByCodeDTO>();
            if (!string.IsNullOrWhiteSpace(userDTO.BindAssistantUserCode))
            {
                var assistant = await _userDBService.FindUserByCodeAsync(userDTO.BindAssistantUserCode);
                userInfo.BindAssistantUserName = assistant.FullName ?? assistant.UserName;
            }
            if (!string.IsNullOrWhiteSpace(userDTO.BindAssistantDoctorUserCode))
            {
                var assistantDoctor = await _userDBService.FindUserByCodeAsync(userDTO.BindAssistantDoctorUserCode);
                userInfo.BindAssistantDoctorUserName = assistantDoctor.FullName ?? assistantDoctor.UserName;
            }
            return userInfo;
        }

        public async Task<bool> IsDefaultPasswordAsync(TokenRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            if (string.IsNullOrWhiteSpace(userInfo.SecretPassword) || userInfo.SecretPassword == "121f69e11038b67debf48095df000d14")//init123456
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 获取用户职称
        /// </summary>
        /// <param name="userList"></param>
        /// <returns></returns>
        private async Task<List<RankDTO>> GetUserRanks(IEnumerable<UserDTO> userList)
        {
            var rankCodes = new List<string>();
            foreach (var item in userList)
            {
                if (item.RankCodes != null && item.RankCodes.Any())
                {
                    rankCodes.AddRange(item.RankCodes);
                }
            }
            if (rankCodes.Any())
            {
                return await _rankDBService.GetAllRanksByOrganizationCodeAsync(rankCodes);
            }
            return new List<RankDTO>();
        }

        private void checkedLoginIsLocked(string userCode)
        {
            var isLocked = _userLoginLock.GetLoginIsLocked(userCode);
            if (isLocked)
            {
                ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
            }
        }

        /// <summary>
        /// 更新用户设置
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>结果</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExisted</errorCodes>
        public async Task<bool> SetCommonSettingsAsync(CommonSettingsRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var findUserDBRequest = new FindUserRequest();
            findUserDBRequest.Code = userCode;
            var srcUserInfoDTO = await _userDBService.FindUserAsync(findUserDBRequest);
            if (srcUserInfoDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotFind, "User not found");
            }
            switch (request.SettingType)
            {
                case CommonSettingsEnum.Signature:
                    if (!string.IsNullOrEmpty(request.Value))
                    {
                        if (!Regex.IsMatch(request.Value, @"[a-zA-z]+://[^\s]*"))
                        {
                            ThrowCustomerException(CustomerRpcCode.SignatureUrlNotValid, "Signature is not valid");
                        }
                        srcUserInfoDTO.Signature = request.Value.ToUrlToken();
                    }
                    break;
                case CommonSettingsEnum.Language:
                    srcUserInfoDTO.Language = request.Value;
                    break;
                case CommonSettingsEnum.EnableReportLabel:
                    Boolean.TryParse(request.Value, out bool enableReportLabel);
                    srcUserInfoDTO.EnableReportLabel = enableReportLabel;
                    break;
                default:
                    return false;
            }
            var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), string.Empty, true);
            return result;
        }

        /// <summary>
        /// 查找日程数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<ClientScheduleDTO>> FindSchedulesAsync(FindSchedulesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (request.StartTime < new DateTime(2000, 1, 1) || request.EndTime < new DateTime(2000, 1, 1))
            {
                ThrowCustomerException(CustomerRpcCode.ExamTimeIsEmpty, "Time is empty");
            }
            var schedules=  await  _userDBService.GetScheduleDBAsync(new GetScheduleDBRequest()
            {
                UserCode=userCode,
                StartTime =request.StartTime,
                EndTime = request.EndTime
            });
           return schedules?.MappingTo<List<ClientScheduleDTO>>();
        }

        private bool GetGradingProtectionConfigCache(string code)
        {
            var item = CacheMaintenance.Instance.Get<IGradingProtectionConfigManager>().Get(code);
            if(item != null)
            {
                if(item.Value.ToLower() == "true")
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// 查询消息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<MessageExtendInfoDTO>> GetMessagesPageByCodeDBAsync(QueryMessageListRequest request)
        {
            var clientId = await GetClientIdByTokenAsync(request.Token);
             var req = new GetMessagesDBPageRequest() 
            {
                ClientId = clientId,
                Status = request.Status,
                Keyword = request.Keyword,
                StartTime = request.StartTime,
                EndTime = request.EndTime,
                NotificationType = request.NotificationType,
                RelevanceCode = request.RelevanceCode,
                PageSize = request.PageSize,
                CurrentPage = request.PageIndex
            };
            var pageResult = await _messageDBService.GetMessagesPageDBAsync(req);
            var result = new PageResult<MessageExtendInfoDTO>();
            if (pageResult != null && pageResult.PageData != null && pageResult.PageData.Count > 0)
            {
                result.PageIndex = pageResult.PageIndex;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.TotalCount;
                var list = new List<MessageExtendInfoDTO>();
                foreach(var dataItem in pageResult.PageData)
                {
                    var clientInfoDTO = dataItem.ClientInfos.FirstOrDefault(c => c.ClientId == clientId) ?? new ClientInfoDTO();
                    var messageExtendInfo = new MessageExtendInfoDTO()
                    {
                        MessageCode = dataItem.Code,
                        NotificationType = dataItem.NotificationType,
                        Content = dataItem.Content,
                        NotifyTime = dataItem.NotifyTime,
                        ReceiverType = dataItem.ReceiverType,
                        RelevanceCode = dataItem.RelevanceCode,
                        IsReaded = clientInfoDTO.IsReaded,
                        DeliveryTime = clientInfoDTO.DeliveryTime,
                        ReadTime = clientInfoDTO.ReadTime
                    };
                    list.Add(messageExtendInfo);
                }
                result.PageData = list;
            }
            return result;
        }

        /// <summary>
        /// 消息送达回调接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> SetMessageDeliveryAsync(SetMessageDeliveryRequest request)
        {
            if (string.IsNullOrEmpty(request.MessageCode))
            {
                ThrowCustomerException(CustomerRpcCode.MessageCodeIsEmpty, "MessageCode is empty");
            }
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var req = new UpdateMessagesDBRequest() 
            {
                ClientId = clientId,
                MessageCode = request.MessageCode,
                IsReaded = request.IsReaded
            };
            var result = await _messageDBService.UpdateMessageInfoDBAsync(req);
            return result;
        }

        /// <summary>
        /// 设置消息为已读
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> BatchSetMessageStatusAsync(SetMessageInfoReqeust request)
        {
            if (request.MessageCodes == null || request.MessageCodes.Count <= 0)
            {
                ThrowCustomerException(CustomerRpcCode.MessageCodesIsEmpty, "MessageCodes is empty");
            }
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var req = new UpdateMessagesStatusDBequest() 
            {
                ClientId = clientId,
                MessageCodes = request.MessageCodes,
            };
            var result = await _messageDBService.UpdateMessagesStatusDBAsync(req);
            return result;
        }

        public async Task<long> GetActiveUsersCount()
        {
            return await _userDBService.GetActiveUsersCount();
        }

        /// <summary>
        /// 获取公告内容列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<AnnouncementExtendInfoDTO>> QueryAnnouncementsByPageAsync(QueryAnnouncementListRequest request)
        {
            if (string.IsNullOrEmpty(request.Language))
            {
                request.Language = "zh-CN";
            }
            var req = new QueryAnnouncementsDBRequest()
            {
                AnnouncementStatus = AnnouncementStatusEnum.Released,
                AnnouncementType = request.AnnouncementType,
                Language = request.Language,
                Keyword = request.Keyword,
                EndTime = request.EndTime,
                StartTime = request.StartTime,
                PageSize = request.PageSize,
                CurrentPage = request.PageIndex
            };
            var pageResult = await _announcementDBService.QueryAnnouncementsDBByPageAsync(req);
            var result = new PageResult<AnnouncementExtendInfoDTO>();
            if (pageResult != null && pageResult.PageData != null && pageResult.PageData.Count > 0)
            {
                result.PageIndex = pageResult.PageIndex;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.TotalCount;
                var list = new List<AnnouncementExtendInfoDTO>();
                foreach(var dataItem in pageResult.PageData)
                {
                    var languageConfig = dataItem.LanguageConfigs?.FirstOrDefault(c => c.Language == request.Language) ?? new AnnouncementLanguageConfigDTO();
                    var announcementExtendInfo = new AnnouncementExtendInfoDTO()
                    {
                        AnnouncementCode = dataItem.Code,
                        AnnouncementType = dataItem.AnnouncementType,
                        Language = languageConfig.Language,
                        Title = languageConfig.Title,
                        Content = languageConfig.Content,
                    };
                    list.Add(announcementExtendInfo);
                }
                result.PageData = list;
            }
            return result;
        }

        /// <summary>
        /// 根据公告Code获取内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<AnnouncementExtendInfoDTO> GetAnnouncementInfoAsync(GetAnnouncementRequest request)
        {
            if (string.IsNullOrEmpty(request.AnnouncementCode))
            {
                ThrowCustomerException(CustomerRpcCode.AnnouncementCodeIsEmpty, "Announcement code is empty");
            }
            if (string.IsNullOrEmpty(request.Language))
            {
                ThrowCustomerException(CustomerRpcCode.LanguageCodeIsEmpty, "Language code is empty");
            }
            var req = new GetAnnouncementDBRequest()
            {
                AnnouncementCode = request.AnnouncementCode
            };
            var findEntity = await _announcementDBService.GetAnnouncementDBAsync(req);
            var result = new AnnouncementExtendInfoDTO();
            if (findEntity != null && !string.IsNullOrEmpty(findEntity.Code))
            {
                var languageConfig = findEntity.LanguageConfigs?.FirstOrDefault(c => c.Language == request.Language) ?? new AnnouncementLanguageConfigDTO();
                result = new AnnouncementExtendInfoDTO()
                {
                    AnnouncementCode = findEntity.Code,
                    AnnouncementType = findEntity.AnnouncementType,
                    Language = languageConfig.Language,
                    Title = languageConfig.Title,
                    Content = languageConfig.Content
                };                
            }
            return result;
        }
    }
}
